#!/usr/bin/ruby

require 'json'
require 'date'
require 'open-uri'
require 'optparse'
require 'socket'
require 'syslog/logger'

path=ENV['PATH']
ENV['PATH'] = "#{path}:/usr/bin:/bin:/opt/pbs/bin"

ENV['PBSPRO_IGNORE_KERBEROS']='yes'

SUFFIX_MAG = { 'kb' => 1024, 'mb' => 1024*1024, 'gb' => 1024*1024*1024, 'tb' => 1024*1024*1024*1024 }

$output = []

$time_now_sec = Time.now.to_i

$hostname = Socket.gethostname

$log = Syslog::Logger.new 'pbs_livejobs.rb'

$affiliation_file = nil

OptionParser.new do |opts|
  opts.banner = "Usage: pbs_livejobs.rb [options]"

  opts.on("-aNAME", "--affiliation-file=NAME", "Affiliation mapping location (fullpath with filename)") do |v|
    $affiliation_file = v
  end
end.parse!

exit 0 if $affiliation_file == nil

orgs = open(URI('https://metavo.metacentrum.cz/pbsmon2/api/org_names'))

exit 0 if orgs == nil;

orgs_a = JSON.parse(File.read(orgs.path).force_encoding('UTF-8'))

orgs_h = {}

orgs_a.each do |org|
  orgs_h[org['user_org']] = org['name']
end

begin
   mappings = JSON.parse(File.read($affiliation_file).force_encoding('UTF-8'))
rescue StandardError => msg
   $log.error("affiliation file error: #{msg}")
   exit 0
end

af = {}

mappings.each do |item|
   login = item['urn:perun:user_facility:attribute-def:virt:login']
   workplaceid = item['urn:perun:user:attribute-def:def:workplaceId']
   facultyid = item['urn:perun:user:attribute-def:def:enrolledFacultyID']
   deforg = item['urn:perun:user:attribute-def:def:organization']
   if item['urn:perun:user:attribute-def:virt:userOrganizations'] != nil
      metaorg = item['urn:perun:user:attribute-def:virt:userOrganizations']['meta']
      einfraorg = item['urn:perun:user:attribute-def:virt:userOrganizations']['einfra']
      elixirorg = item['urn:perun:user:attribute-def:virt:userOrganizations']['elixir-cz']
   end
   if metaorg != nil
      wplace = metaorg
   elsif einfraorg != nil
      wplace = einfraorg
   elsif elixirorg != nil
      wplace = elixirorg
   else
      wplace = deforg
   end

   if orgs_h[wplace] == nil
      wplace = 'Others'
   else
      wplace = orgs_h[wplace]
   end
   if login =~ /^9....../
      wplace = 'Guests'
   end
   if workplaceid != nil
      wplace = workplaceid
   elsif facultyid != nil
      wplace = facultyid
   end
   af[login] = wplace if af[login] == nil
end

time_used = {}

qinfo_a = []

begin
  qinfo = ((`qstat -t -f -F dsv`).force_encoding('UTF-8'))
rescue StandardError => msg
  $log.error("qstat error: #{msg}")
  exit 0
end

qinfo.each_line do |line| 
  job = {}
  items = line.encode('UTF-8', :invalid => :replace).split('|')
  job['id'] = items[0].sub(/Job Id: /, '')
  items.each do |it|
    k,v = it.split('=',2)
    if (['Job_Name', 'job_state', 'queue', 'Job_Owner', 'exec_host', 'Resource_List.mem', 'Resource_List.ncpus', 'Resource_List.ngpus', 
         'Resource_List.walltime', 'Resource_List.scratch_local', 'Resource_List.scratch_ssd']).include? k
      job[k] = v
    end
  end
  qinfo_a << job
end

qinfo_a.each do |job|
   job_state = job['job_state']
   queue = job['queue']
   user = job['Job_Owner'].gsub(/@.*/, '')
   if job['exec_host'] != nil
      hosts = job['exec_host'].gsub(/\/[0-9\*]*/, '').split('+').join(':')
   else
      hosts = ''
   end
   wid = nil
   wid = af[user].gsub(/ /, '_') if af[user] != nil
   wid = 'nil' if wid == nil
   ncpus = 0
   ngpus = 0
   walltime = 0
   mem = 0
   scratch = 0
   if job['Resource_List.ncpus'] != nil
      ncpus = job['Resource_List.ncpus']
      ngpus = job['Resource_List.ngpus']
      _tmp = job['Resource_List.walltime']
      walltime = (_tmp[0].to_i)*3600+(_tmp[1].to_i)*60+(_tmp[2].to_i)
      _, val, mag = job['Resource_List.mem'].match(/^([0-9]*)([a-z]*)/).to_a
      mem = val.to_i
      mem = mem * SUFFIX_MAG[mag].to_i if mag != nil
      scratch_local = 0
      if job['Resource_List.scratch_local'] != nil
        _, val, mag = job['Resource_List.scratch_local'].match(/^([0-9]*)([a-z]*)/).to_a
        scratch_local = val.to_i
        scratch_local = scratch_local * SUFFIX_MAG[mag].to_i if mag != nil
      end
      scratch_ssd = 0
      if job['Resource_List.scratch_ssd'] != nil
        _, val, mag = job['Resource_List.scratch_ssd'].match(/^([0-9]*)([a-z]*)/).to_a
        scratch_ssd = val.to_i
        scratch_ssd = scratch_ssd * SUFFIX_MAG[mag].to_i if mag != nil
      end
      scratch = scratch_ssd + scratch_local
   end
   time_usec = $time_now_sec.to_i * 1000000
   while time_used[time_usec] != nil
     time_usec += 1
   end
   time_used[time_usec] = 1
   $output << { 'name' => 'livejobs',
                # tags
                'host'      => $hostname,
                'job_state' => job_state,
                'queue'     => queue,
                'user'      => user,
                'wid'       => wid,
                'hosts'     => hosts,
		'job_name'  => job['Job_Name'],
		'job_id'    => job['id'],
                # vals
                'ncpus'     => ncpus.to_i,
                'ngpus'     => ngpus.to_i,
                'walltime'  => walltime,
                'mem'       => mem, 
                'scratch'   => scratch,
                'time'      => time_usec}      
end

puts(JSON.pretty_generate($output))
